@@ -13,13 +13,13 @@
 #include <libintl.h>
 #include <sys/types.h>
 #include <pwd.h>
-#include <dirent.h>
 #include <stdio.h>
 #include <sys/stat.h>
 #include <string.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <libgen.h>
+#include <gdk/gdkx.h>
 #include <libgnomevfs/gnome-vfs.h>
 #include <libgnomevfs/gnome-vfs-mime.h>
 #include <libgnomevfs/gnome-vfs-mime-handlers.h>
@@ -31,10 +31,8 @@
 #include <libgnomevfs/gnome-vfs-types.h>
 #include <libgnomevfs/gnome-vfs-application-registry.h>
 
-
 #include <gpe/init.h>
 #include <gpe/errorbox.h>
-#include <gpe/render.h>
 #include <gpe/pixmaps.h>
 #include <gpe/picturebutton.h>
 #include <gpe/question.h>
@@ -42,10 +40,10 @@
 #include <gpe/dirbrowser.h>
 #include <gpe/spacing.h>
 #include <gpe/gpehelp.h>
+#include <X11/Xlib.h>
+#include <gpe/infoprint.h>
 
-#include "mime-sql.h"
-#include "mime-programs-sql.h"
-
+#include "guitools.h"
 #include "bluetooth.h"
 
 #define _(x) dgettext(PACKAGE, x)
@@ -98,7 +96,7 @@ static GtkWidget *view_widget;
 static GtkWidget *dir_view_widget;
 static GtkWidget *view_window;
 static GtkWidget *dir_view_window;
-static GtkWidget *vbox, *main_hbox;
+static GtkWidget *vbox, *main_paned;
 
 static GtkWidget *bluetooth_menu_item;
 static GtkWidget *copy_menu_item;
@@ -110,10 +108,11 @@ static GtkWidget *properties_menu_item;
 static GtkWidget *delete_menu_item;
 static GtkWidget *set_dirbrowser_menu_item;
 static GtkWidget *set_myfiles_menu_item;
+static GtkWidget *goto_menu_item;
 
 static GtkWidget *btnListView;
 static GtkWidget *btnIconView;
-static GtkWidget *btnGoUp;
+static GtkWidget *btnGoUp = NULL;
 
 GdkPixbuf *default_pixbuf;
 
@@ -178,7 +177,7 @@ struct gpe_icon my_icons[] = {
 
 /* some forward declarations */
 
-static void browse_directory (gchar *directory);
+void browse_directory (gchar *directory);
 static void popup_ask_open_with (void);
 static void popup_ask_move_file (void);
 static void popup_ask_rename_file (void);
@@ -224,6 +223,7 @@ static GtkItemFactoryEntry mMain_items[] = {
   { N_("/_Settings"),         NULL,         NULL, 0, "<Branch>" },
   { N_("/_Settings/Directory Browser"), NULL, on_dirbrowser_setting_changed, 0, "<CheckItem>"},
   { N_("/_Settings/My Files only"), NULL, on_myfiles_setting_changed, 0, "<CheckItem>"},
+  { N_("/_Go To"),         NULL,         NULL,           0, "<Branch>" },
   { N_("/_Help"),         NULL,         NULL,           0, "<Branch>" },
   { N_("/_Help/Index"),   NULL,         on_help_clicked,    0, "<StockItem>",GTK_STOCK_HELP },
   { N_("/_Help/About"),   NULL,         on_about_clicked,    0, "<Item>" },
@@ -250,8 +250,10 @@ create_mMain(GtkWidget  *window)
     gtk_item_factory_get_item (itemfactory, N_("/Settings/Directory Browser"));
   set_myfiles_menu_item = 
     gtk_item_factory_get_item (itemfactory, N_("/Settings/My Files only"));
-
-  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(set_dirbrowser_menu_item),
+  goto_menu_item = 
+    gtk_item_factory_get_item (itemfactory, N_("/Go To"));
+  
+ gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(set_dirbrowser_menu_item),
                                  directory_browser); 
   
   return (gtk_item_factory_get_widget (itemfactory, "<main>"));
@@ -332,7 +334,8 @@ progress_dialog_create(gchar *title)
          gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox),
                             progress,FALSE,TRUE,0);
          g_object_set_data(G_OBJECT(dialog),"bar",progress);
-         g_signal_connect(G_OBJECT(dialog),"response",G_CALLBACK(progress_response),NULL);
+         g_signal_connect(G_OBJECT(dialog),"response",
+                          G_CALLBACK(progress_response),NULL);
          gtk_widget_show_all(dialog);
   return dialog;
 }
@@ -555,7 +558,11 @@ create_directory_interactive(void)
     response = gnome_vfs_make_directory(diruri,S_IRWXU);
     if (response != GNOME_VFS_OK)
       gpe_error_box (gnome_vfs_result_to_string(response));
-    refresh_current_directory();
+    else
+      {
+        gpe_popup_infoprint(GDK_DISPLAY(), _("Directory created."));
+        refresh_current_directory();
+      }
   }
   gtk_widget_destroy(dialog);
 }
@@ -579,11 +586,15 @@ copy_file (const gchar *src_uri_txt, const gchar *dest_uri_txt)
                                (GnomeVFSXferProgressCallback) transfer_callback,
                                NULL);
   if ((result != GNOME_VFS_OK) && (result != GNOME_VFS_ERROR_INTERRUPTED))
-  {
-    error = g_strdup_printf ("Error: %s", gnome_vfs_result_to_string(result));
-    gpe_error_box (error);
-    g_free (error);
-  }
+    {
+      error = g_strdup_printf ("Error: %s", gnome_vfs_result_to_string(result));
+      gpe_error_box (error);
+      g_free (error);
+    }
+  else
+    {
+      gpe_popup_infoprint(GDK_DISPLAY(), _("File copied."));
+    }
   gnome_vfs_uri_unref(uri_src);
   gnome_vfs_uri_unref(uri_dest);
 }
@@ -598,6 +609,7 @@ copy_file_clip (void)
       gnome_vfs_get_uri_from_local_path(current_popup_file->filename);
   else
     file_clipboard = g_strdup(current_popup_file->filename);
+  gpe_popup_infoprint(GDK_DISPLAY(), _("File copied to clipboard."));
 }
 
 
@@ -665,9 +677,15 @@ open_with (GnomeVFSMimeApplication *application,
            FileInformation *file_info)
 {
   pid_t pid;
-
+    
   if (application)
   {
+    gchar *msg = g_strdup_printf("%s %s",_("Starting"),application->name);
+    Display *dpy = GDK_DISPLAY();
+      
+    gpe_popup_infoprint(dpy, msg);
+    g_free(msg);
+      
 	pid = fork();
 	switch (pid)
 	{
@@ -692,16 +710,7 @@ open_with (GnomeVFSMimeApplication *application,
   }
 }
 
-/*static void
-open_with_row_selected (GtkCList *clist, gint row, gint column, GdkEventButton *event, gpointer entry)
-{
-  GnomeVFSMimeApplication *application;
 
-  application = gtk_clist_get_row_data (GTK_CLIST (clist), row);
-  gtk_object_set_data (GTK_OBJECT (entry), "GnomeVFSMimeApplication", (gpointer) application);
-  gtk_entry_set_text (GTK_ENTRY (entry), application->name);
-}
-*/
 static void
 rename_file (GtkWidget *dialog_window, gint response_id)
 {
@@ -709,40 +718,43 @@ rename_file (GtkWidget *dialog_window, gint response_id)
   GnomeVFSResult result;
 
   if (response_id == GTK_RESPONSE_ACCEPT)
-  {
-  gchar *tmp = g_strdup (current_popup_file->filename);
-  dest = g_strdup_printf ("%s/%s", dirname (tmp), gtk_entry_get_text (GTK_ENTRY (g_object_get_data (G_OBJECT (dialog_window), "entry"))));
-  g_free (tmp);
-  result = gnome_vfs_move_uri (gnome_vfs_uri_new (current_popup_file->filename), gnome_vfs_uri_new (dest), TRUE);
-
-  if (result != GNOME_VFS_OK)
-  {
-    switch (result)
     {
-    case GNOME_VFS_ERROR_READ_ONLY:
-      error = g_strdup ("File is read only.");
-      break;
-    case GNOME_VFS_ERROR_ACCESS_DENIED:
-      error = g_strdup ("Access denied.");
-      break;
-    case GNOME_VFS_ERROR_READ_ONLY_FILE_SYSTEM:
-      error = g_strdup ("Read only file system.");
-      break;
-    case GNOME_VFS_ERROR_FILE_EXISTS:
-      error = g_strdup ("Destination file already exists.");
-      break;
-    default:
-      error = g_strdup_printf ("Error: %s", gnome_vfs_result_to_string(result));
-      break;
+      gchar *tmp = g_strdup (current_popup_file->filename);
+      dest = g_strdup_printf ("%s/%s", dirname (tmp), gtk_entry_get_text (GTK_ENTRY (g_object_get_data (G_OBJECT (dialog_window), "entry"))));
+      g_free (tmp);
+      result = gnome_vfs_move_uri (gnome_vfs_uri_new (current_popup_file->filename), gnome_vfs_uri_new (dest), TRUE);
+    
+      if (result != GNOME_VFS_OK)
+      {
+        switch (result)
+          {
+          case GNOME_VFS_ERROR_READ_ONLY:
+            error = g_strdup ("File is read only.");
+            break;
+          case GNOME_VFS_ERROR_ACCESS_DENIED:
+            error = g_strdup ("Access denied.");
+            break;
+          case GNOME_VFS_ERROR_READ_ONLY_FILE_SYSTEM:
+            error = g_strdup ("Read only file system.");
+            break;
+          case GNOME_VFS_ERROR_FILE_EXISTS:
+            error = g_strdup ("Destination file already exists.");
+            break;
+          default:
+            error = g_strdup_printf ("Error: %s", 
+                                     gnome_vfs_result_to_string(result));
+            break;
+          }
+          gpe_error_box (error);
+          g_free (error);
+        }
+      else
+        {
+          gpe_popup_infoprint(GDK_DISPLAY(), _("File renamed."));
+        }
+      g_free (dest);
     }
 
-    gpe_error_box (error);
-    g_free (error);
-  }
-
-  g_free (dest);
-  }
-
   gtk_widget_destroy (dialog_window);
   refresh_current_directory();
 }
@@ -758,32 +770,33 @@ move_file (gchar *directory)
   result = gnome_vfs_move_uri (gnome_vfs_uri_new (current_popup_file->filename), gnome_vfs_uri_new (dest), TRUE);
 
   if (result != GNOME_VFS_OK)
-  {
-    switch (result)
     {
-    case GNOME_VFS_ERROR_NO_SPACE:
-      error = g_strdup ("No space left on device.");
-      break;
-    case GNOME_VFS_ERROR_READ_ONLY:
-      error = g_strdup ("Destination is read only.");
-      break;
-    case GNOME_VFS_ERROR_ACCESS_DENIED:
-      error = g_strdup ("Access denied.");
-      break;
-    case GNOME_VFS_ERROR_READ_ONLY_FILE_SYSTEM:
-      error = g_strdup ("Read only file system.");
-      break;
-    default:
-      error = g_strdup_printf ("Error: %s", gnome_vfs_result_to_string(result));
-      break;
+      switch (result)
+      {
+      case GNOME_VFS_ERROR_NO_SPACE:
+        error = g_strdup ("No space left on device.");
+        break;
+      case GNOME_VFS_ERROR_READ_ONLY:
+        error = g_strdup ("Destination is read only.");
+        break;
+      case GNOME_VFS_ERROR_ACCESS_DENIED:
+        error = g_strdup ("Access denied.");
+        break;
+      case GNOME_VFS_ERROR_READ_ONLY_FILE_SYSTEM:
+        error = g_strdup ("Read only file system.");
+        break;
+      default:
+        error = g_strdup_printf ("Error: %s", gnome_vfs_result_to_string(result));
+        break;
+      }
+      gpe_error_box (error);
+      g_free (error);
     }
-
-    gpe_error_box (error);
-    g_free (error);
-  }
   else
-    refresh_current_directory();
-
+    {
+      gpe_popup_infoprint(GDK_DISPLAY(), _("File moved."));
+      refresh_current_directory();
+    }
   g_free (dest);
 }
 
@@ -808,10 +821,19 @@ popup_ask_rename_file ()
 
   label_text = g_strdup_printf ("Rename file %s to:", current_popup_file->vfs->name);
 
-  dialog_window = gtk_dialog_new_with_buttons ("Rename file", GTK_WINDOW (window), GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT, GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT, GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL);
-  g_signal_connect (G_OBJECT (dialog_window), "delete_event", GTK_SIGNAL_FUNC (kill_widget), dialog_window);
-  g_signal_connect (G_OBJECT (dialog_window), "response", GTK_SIGNAL_FUNC (rename_file), NULL);
-
+  dialog_window = gtk_dialog_new_with_buttons ("Rename file", 
+                                               GTK_WINDOW (window), 
+                                               GTK_DIALOG_MODAL 
+                                                 | GTK_DIALOG_DESTROY_WITH_PARENT, 
+                                               GTK_STOCK_CANCEL, 
+                                               GTK_RESPONSE_REJECT, 
+                                               GTK_STOCK_OK, 
+                                               GTK_RESPONSE_ACCEPT, NULL);
+  g_signal_connect (G_OBJECT (dialog_window), "delete_event", 
+                    G_CALLBACK (kill_widget), dialog_window);
+  g_signal_connect (G_OBJECT (dialog_window), "response", 
+                    G_CALLBACK (rename_file), NULL);
+  
   vbox = gtk_vbox_new (FALSE, 0);
 
   label = gtk_label_new (label_text);
@@ -1034,31 +1056,6 @@ ask_open_with (FileInformation *file_info)
     
   gtk_widget_destroy(dialog_window);
   g_list_free(appnames);
-    
-/*
-  if (mime_programs)
-  {
-    for (iter = mime_programs; iter; iter = iter->next)
-    {
-      struct mime_program *program = iter->data;
-
-      row_text[0] = "";
-      row_text[1] = program->name;
-      gtk_clist_append (GTK_CLIST (clist), row_text);
-      gtk_clist_set_row_data (GTK_CLIST (clist), row_num, (gpointer) program->command);
-
-      pixmap_file = g_strdup_printf ("%s/share/pixmaps/%s.png", PREFIX, program->command);
-      pixbuf = gdk_pixbuf_new_from_file (pixmap_file, NULL);
-      if (pixbuf != NULL)
-      {
-        spixbuf = gdk_pixbuf_scale_simple (pixbuf, 12, 12, GDK_INTERP_BILINEAR);
-        gpe_render_pixmap (NULL, spixbuf, &pixmap, &bitmap);
-        gtk_clist_set_pixmap (GTK_CLIST (clist), row_num, 0, pixmap, bitmap);
-      }
-      row_num++;
-    }
-  }
-*/
 }
 
 static void
@@ -1119,7 +1116,6 @@ button_clicked (GtkWidget *widget, gpointer udata)
 {
   GnomeVFSMimeApplication *default_mime_application;
   FileInformation *file_info;
-  pid_t pid;
     
   file_info = (FileInformation *) udata;
 
@@ -1163,21 +1159,7 @@ button_clicked (GtkWidget *widget, gpointer udata)
           default_mime_application = gnome_vfs_mime_get_default_application (file_info->vfs->mime_type);
           if (default_mime_application != NULL)
           {
-			pid = fork();
-			switch (pid)
-			{
-				case -1: 
-					return; /* failed */
-				break;
-				case  0: 
-				  if (default_mime_application->requires_terminal)
-					execlp(DEFAULT_TERMINAL,DEFAULT_TERMINAL,default_mime_application->command,file_info->filename);
-				  else
-					execlp(default_mime_application->command,default_mime_application->command,file_info->filename,NULL);
-				break;
-				default: 
-				break;
-			} 
+            open_with(default_mime_application, file_info);
           }
           else
             ask_open_with (file_info);
@@ -1489,14 +1471,16 @@ directory_changed (GnomeVFSMonitorHandle *handle,
     }
 }
 
-static void
+void
 browse_directory (gchar *directory)
 {
   gboolean enabled;
   /* disable "up" button if we are in homedir and view is limited */
-  enabled = (!limited_view || strcmp(directory,g_get_home_dir()));
-  gtk_widget_set_sensitive(btnGoUp,enabled);
-  
+  enabled = (!limited_view || strcmp(directory, g_get_home_dir()));
+  if (btnGoUp)
+    gtk_widget_set_sensitive(btnGoUp,enabled);
+  set_active_item(directory);  
+
   /* some hacks to handle nasty smb urls */
   if (g_str_has_prefix(directory,"smb:"))
   {
@@ -1681,7 +1665,9 @@ tree_button_press (GtkWidget *tree,GdkEventButton *b, gpointer user_data)
 
 	    gtk_tree_model_get_iter (GTK_TREE_MODEL (store), &iter, path);
 
-	    gtk_tree_model_get (GTK_TREE_MODEL (store), &iter, tree == view_widget ? COL_DATA : COL_DIRDATA, &i, -1);
+	    gtk_tree_model_get (GTK_TREE_MODEL (store), &iter, 
+                            tree == view_widget ? COL_DATA : COL_DIRDATA, 
+                            &i, -1);
 		
 	    gtk_tree_path_free (path);
         if (b->button == 3) 
@@ -1788,7 +1774,7 @@ create_view_widget_list(void)
     gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(view_window),
   		GTK_POLICY_NEVER,GTK_POLICY_AUTOMATIC);
     gtk_container_add(GTK_CONTAINER(view_window),treeview);
-    gtk_box_pack_end (GTK_BOX (main_hbox), view_window, TRUE, TRUE, 0);
+    gtk_paned_pack2(GTK_PANED (main_paned), view_window, TRUE, TRUE);
 
 	g_signal_connect (G_OBJECT(treeview), "button_press_event", 
 		G_CALLBACK(tree_button_press), NULL);
@@ -1824,7 +1810,7 @@ create_view_widget_icons(void)
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(view_window),
   	GTK_POLICY_NEVER,GTK_POLICY_AUTOMATIC);
   gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(view_window),view_icons);
-  gtk_box_pack_end (GTK_BOX (main_hbox), view_window, TRUE, TRUE, 0);
+  gtk_paned_pack2(GTK_PANED (main_paned), view_window, TRUE, TRUE);
   
   gtk_widget_show_all(view_window);  
   return view_icons;
@@ -1858,6 +1844,11 @@ create_dir_view_widget(void)
 							   "text",
 							   COL_DIRNAME,
 							   NULL);
+    /* min width = 100 pixels */
+    gtk_tree_view_column_set_min_width(column, 100);
+    /* max width = 50% */
+    gtk_tree_view_column_set_max_width(column, gdk_screen_width() / 2);
+      
 	gtk_tree_view_column_set_resizable(GTK_TREE_VIEW_COLUMN(column),TRUE);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
 
@@ -1865,8 +1856,8 @@ create_dir_view_widget(void)
     gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(dir_view_window),
   		GTK_POLICY_NEVER,GTK_POLICY_AUTOMATIC);
     gtk_container_add(GTK_CONTAINER(dir_view_window),treeview);
-    gtk_box_pack_start (GTK_BOX (main_hbox), dir_view_window, FALSE, TRUE, 0);
-
+    gtk_paned_pack1 (GTK_PANED(main_paned), dir_view_window, FALSE, TRUE);
+    
 	g_signal_connect (G_OBJECT(treeview), "button_press_event", 
 		G_CALLBACK(tree_button_press), NULL);
 	g_signal_connect (G_OBJECT(treeview), "button_release_event", 
@@ -1884,6 +1875,7 @@ main (int argc, char *argv[])
   GdkPixbuf *p;
   GtkWidget *pw;
   GtkAccelGroup *accel_group;
+  GtkWidget *storage_menu;
   int size_x, size_y;
 
   if (gpe_application_init (&argc, &argv) == FALSE)
@@ -1898,6 +1890,8 @@ main (int argc, char *argv[])
   
   bluetooth_init ();
 
+  gnome_vfs_init ();
+  
   /* init tree storage stuff */
   store = gtk_tree_store_new (N_COLUMNS,
                               G_TYPE_OBJECT,
@@ -1932,7 +1926,7 @@ main (int argc, char *argv[])
 
   gtk_widget_realize (window);
 
-  main_hbox = gtk_hbox_new (FALSE, gpe_get_boxspacing());
+  main_paned = gtk_hpaned_new ();
   
   vbox = gtk_vbox_new (FALSE, 0);
 
@@ -1948,9 +1942,11 @@ main (int argc, char *argv[])
     "activate", GTK_SIGNAL_FUNC (goto_directory), NULL);
   
   view_widget = create_view_widget_list();
-  
+#warning necessary?  
   dir_view_widget = create_dir_view_widget();
   
+  storage_menu = build_storage_menu(directory_browser);
+  
   toolbar = gtk_toolbar_new ();
   gtk_toolbar_set_orientation (GTK_TOOLBAR (toolbar), GTK_ORIENTATION_HORIZONTAL);
 
@@ -1958,25 +1954,32 @@ main (int argc, char *argv[])
   gtk_toolbar_set_orientation (GTK_TOOLBAR (toolbar2), GTK_ORIENTATION_HORIZONTAL);
 
   gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar), GTK_STOCK_GO_BACK,
-			    _("Back"), _("Go back in history."),
+			    _("Go back in history."), NULL,
 			    G_CALLBACK (history_back), NULL, -1);
 
   gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar), GTK_STOCK_GO_FORWARD,
-			    _("Forward"), _("Go forward in history."),
+			    _("Go forward in history."), NULL,
 			    G_CALLBACK (history_forward), NULL, -1);
 
   btnGoUp = gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar), GTK_STOCK_GO_UP,
-			                          _("Up one level"), _("Go up one level."),
+			                          _("Go up one level."), NULL,
 			                          G_CALLBACK (up_one_level), NULL, -1);
 
   gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));
 
-  gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar), GTK_STOCK_HOME,
-			    _("Home"), _("Goto your home directory."),
-			    G_CALLBACK (set_directory_home), NULL, -1);
+  if (!directory_browser)
+    {
+      gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar), GTK_STOCK_HOME,
+	                            _("Goto your home directory."), NULL,
+                                G_CALLBACK (set_directory_home), NULL, -1);
+    }
+                
+  gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar), GTK_STOCK_REFRESH,
+			    _("Refresh current directory."), NULL,
+			    G_CALLBACK (refresh_current_directory), NULL, -1);
                 
   gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar), GTK_STOCK_STOP,
-			    _("Stop"), _("Stop the current process."),
+			    _("Stop the current process."), NULL,
 			    G_CALLBACK (safety_check), NULL, -1);
 
   gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));
@@ -1984,28 +1987,30 @@ main (int argc, char *argv[])
   p = gpe_find_icon ("icon-view");
   pw = gtk_image_new_from_pixbuf(p);
   btnIconView = gtk_toolbar_append_item (GTK_TOOLBAR (toolbar), _("Icon view"), 
-			   _("Icon view"), _("View files as icons"), pw, 
+			   _("View files as icons"), NULL, pw, 
 			   G_CALLBACK (view_icons), NULL);
   
   p = gpe_find_icon ("list-view");
   pw = gtk_image_new_from_pixbuf(p);
   btnListView = gtk_toolbar_append_item (GTK_TOOLBAR (toolbar), _("List view"), 
-			   _("List view"), _("View files in a list"), pw, 
+			   _("View files in a list"), NULL, pw, 
 			   G_CALLBACK (view_list), NULL);
   gtk_widget_set_sensitive(btnListView,FALSE);
                
   pw = gtk_image_new_from_stock(GTK_STOCK_JUMP_TO, 
                                 gtk_toolbar_get_icon_size(GTK_TOOLBAR(toolbar2)));
   gtk_toolbar_append_item (GTK_TOOLBAR (toolbar2), _("Go!"), 
-			   _("Goto Location"), _("Goto Location"), pw, 
+			   _("Goto Location"), NULL, pw, 
 			   G_CALLBACK (goto_directory), NULL);
   
   gtk_container_add (GTK_CONTAINER (window), GTK_WIDGET (vbox));
   gtk_box_pack_start (GTK_BOX (vbox), toolbar, FALSE, FALSE, 0);
+  if (directory_browser)
+    gtk_box_pack_start (GTK_BOX (vbox), storage_menu, FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX (hbox), combo, TRUE, TRUE, 0);
   gtk_box_pack_start (GTK_BOX (hbox), toolbar2, FALSE, FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (vbox), main_hbox, TRUE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (vbox), main_paned, TRUE, TRUE, 0);
 
   gpe_set_window_icon (window, "icon");
 
@@ -2014,7 +2019,10 @@ main (int argc, char *argv[])
   g_object_set_data_full (G_OBJECT (window), "<main>", item_factory, (GDestroyNotify) g_object_unref);
   gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);
   gtk_item_factory_create_items (item_factory, nmenu_items, menu_items, NULL);
-
+  
+  if (!directory_browser)
+    gtk_menu_item_set_submenu(GTK_MENU_ITEM(goto_menu_item), storage_menu);
+  
   bluetooth_menu_item = gtk_item_factory_get_widget (item_factory, "<main>/Send via Bluetooth");
   copy_menu_item = gtk_item_factory_get_widget (item_factory, "<main>/Copy");
   paste_menu_item = gtk_item_factory_get_widget (item_factory, "<main>/Paste");
@@ -2032,7 +2040,6 @@ main (int argc, char *argv[])
 
   gtk_widget_show_all (window);
   
-  gnome_vfs_init ();
   gnome_vfs_module_callback_set_default (GNOME_VFS_MODULE_CALLBACK_AUTHENTICATION,
 						  (GnomeVFSModuleCallback) auth_callback,
 						  NULL,
@@ -2042,7 +2049,10 @@ main (int argc, char *argv[])
   gnome_vfs_application_registry_reload();
   
   if (directory_browser)
-    gtk_widget_show_all(dir_view_window);
+    {
+      gtk_widget_hide(goto_menu_item);
+      gtk_widget_show_all(dir_view_window);
+    }
   else
 	gtk_widget_hide(dir_view_window);
 
